/*
 * Copyright (C) 2008 The Android Open Source Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

 /*
  * This file was autogenerated by irt_syscalls.py
  */
#ifndef _IRT_SYSCALLS_H
#define _IRT_SYSCALLS_H

#include <sys/cdefs_elf.h>
#include <machine/cdefs.h>
#include <string.h>

#include <sys/types.h>
#include <sys/epoll.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <poll.h>
#include <stddef.h>
#include <fcntl.h>
#include <time.h>
#include <nacl_socket.h>
#include <nacl_stat.h>

#ifdef weak_alias
#undef weak_alias
#endif
#define weak_alias(oldname, newname)       \
  extern typeof(oldname) newname  __attribute__ ((weak, alias (#oldname)));

#ifdef libc_hidden_def
#undef libc_hidden_def
#endif
#define libc_hidden_def(x)

struct dirent;
struct epoll_event;
struct msghdr;
struct nacl_abi_stat;
struct sockaddr;
struct timespec;
struct timeval;
struct NaClMemMappingInfo;

#include <irt.h>
#include <irt_dev.h>
#include <irt_poll.h>
#include <irt_socket.h>

typedef void (*nacl_start_func_t)(void);

__BEGIN_DECLS
typedef struct {
  size_t  (*nacl_irt_query)(const char* interface_ident, void* table, size_t tablesize);
  int (*nacl_irt_accept)(int sockfd, struct nacl_abi_sockaddr* addr, size_t addrlen, int* newfd);
  int (*nacl_irt_access)(const char* path, int amode);
  int (*nacl_irt_allocate_code_data)(uintptr_t hint, size_t code_size, uintptr_t data_offset, size_t data_size, uintptr_t* begin);
  int (*nacl_irt_bind)(int sockfd, const struct nacl_abi_sockaddr* local, size_t addrlen);
  int (*nacl_irt_chdir)(const char* pathname);
  int (*nacl_irt_chmod)(const char* path, mode_t mode);
  int (*nacl_irt_clock)(clock_t* ticks);
  int (*nacl_irt_clock_getres)(nacl_irt_clockid_t clock_id, struct timespec* res);
  int (*nacl_irt_clock_gettime)(nacl_irt_clockid_t clock_id, struct timespec* tp);
  int (*nacl_irt_close)(int fd);
  int (*nacl_irt_cond_broadcast)(int cond_handle);
  int (*nacl_irt_cond_create)(int* cond_handle);
  int (*nacl_irt_cond_destroy)(int cond_handle);
  int (*nacl_irt_cond_signal)(int cond_handle);
  int (*nacl_irt_cond_timed_wait_abs)(int cond_handle, int mutex_handle, const struct timespec* abstime);
  int (*nacl_irt_cond_wait)(int cond_handle, int mutex_handle);
  int (*nacl_irt_connect)(int sockfd, const struct nacl_abi_sockaddr* dst, size_t addrlen, int* newfd);
  int (*nacl_irt_dup)(int fd, int* newfd);
  int (*nacl_irt_dup2)(int fd, int newfd);
  int (*nacl_irt_dyncode_create)(void* dest, const void* src, size_t size);
  int (*nacl_irt_dyncode_delete)(void* dest, size_t size);
  int (*nacl_irt_dyncode_modify)(void* dest, const void* src, size_t size);
  int (*nacl_irt_epoll_create)(int size, int* newfd);
  int (*nacl_irt_epoll_ctl)(int epfd, int op, int fd, struct epoll_event* event);
  int (*nacl_irt_epoll_wait)(int epfd, struct epoll_event* events, int maxevents, int timeout);
  int (*nacl_irt_exception_clear_flag)(void);
  int (*nacl_irt_exception_handler)(NaClExceptionHandler handler, NaClExceptionHandler* old_handler);
  int (*nacl_irt_exception_stack)(void* stack, size_t size);
  void (*nacl_irt_exit)(int status);
  int (*nacl_irt_fchdir)(int fd);
  int (*nacl_irt_fchmod)(int fd, mode_t mode);
  int (*nacl_irt_fdatasync)(int fd);
  int (*nacl_irt_fstat)(int fd, struct stat*);
  int (*nacl_irt_fsync)(int fd);
  int (*nacl_irt_ftruncate)(int fd, nacl_irt_int64_t length);
  int (*nacl_irt_futex_wait_abs)(volatile int* addr, int value, const struct timespec* abstime);
  int (*nacl_irt_futex_wake)(volatile int* addr, int nwake, int* count);
  int (*nacl_irt_get_random_bytes)(void* buf, size_t count, size_t* nread);
  int (*nacl_irt_getcwd)(char* pathname, size_t len);
  int (*nacl_irt_getdents)(int fd, struct dirent*, size_t count, size_t* nread);
  int (*nacl_irt_getpeername)(int sockfd, struct nacl_abi_sockaddr* addr, size_t* addrlen);
  int (*nacl_irt_getpid)(int* pid);
  int (*nacl_irt_getsockname)(int sockfd, struct nacl_abi_sockaddr* addr, size_t* addrlen);
  int (*nacl_irt_getsockopt)(int sockfd, int level, int optname, void* optval, size_t* optlen);
  int (*nacl_irt_gettod)(struct timeval* tv);
  int (*nacl_irt_isatty)(int fd, int* result);
  int (*nacl_irt_link)(const char* oldpath, const char* newpath);
  int (*nacl_irt_list_mappings)(struct NaClMemMappingInfo* regions, size_t count, size_t* result_count);
  int (*nacl_irt_listen)(int sockfd, int backlog);
  int (*nacl_irt_lstat)(const char* pathname, struct stat*);
  int (*nacl_irt_mkdir)(const char* pathname, mode_t mode);
  int (*nacl_irt_mmap)(void* *addr, size_t len, int prot, int flags, int fd, nacl_irt_int64_t off);
  int (*nacl_irt_mprotect)(void* addr, size_t len, int prot);
  int (*nacl_irt_munmap)(void* addr, size_t len);
  int (*nacl_irt_mutex_create)(int* mutex_handle);
  int (*nacl_irt_mutex_destroy)(int mutex_handle);
  int (*nacl_irt_mutex_lock)(int mutex_handle);
  int (*nacl_irt_mutex_trylock)(int mutex_handle);
  int (*nacl_irt_mutex_unlock)(int mutex_handle);
  int (*nacl_irt_nanosleep)(const struct timespec* req, struct timespec* rem);
  int (*nacl_irt_open)(const char* pathname, int oflag, mode_t cmode, int* newfd);
  int (*nacl_irt_open_resource)(const char* file, int* fd);
  int (*nacl_irt_pipe)(int fds[2]);
  int (*nacl_irt_poll)(struct NaClPollFD* fds, size_t nfds, int timeout, int* cnt);
  int (*nacl_irt_read)(int fd, void* buf, size_t count, size_t* nread);
  int (*nacl_irt_readlink)(const char* path, char* buf, size_t count, size_t* nread);
  int (*nacl_irt_recvfrom)(int sockfd, void* buffer, size_t len, int flags, struct nacl_abi_sockaddr* src, size_t* addrlen);
  int (*nacl_irt_register_block_hooks)(void (*pre)(void), void (*post)(void));
  int (*nacl_irt_rename)(const char* oldpath, const char* newpath);
  int (*nacl_irt_rmdir)(const char* pathname);
  int (*nacl_irt_sched_yield)(void);
  int (*nacl_irt_seek)(int fd, nacl_irt_int64_t offset, int whence, nacl_irt_int64_t* new_offset);
  int (*nacl_irt_select)(size_t nfds, struct NaClFDSet* readfds, struct NaClFDSet* writefds, struct NaClFDSet* exceptfds, struct timeval* timeout);
  int (*nacl_irt_sem_create)(int* sem_handle, int32_t value);
  int (*nacl_irt_sem_destroy)(int sem_handle);
  int (*nacl_irt_sem_post)(int sem_handle);
  int (*nacl_irt_sem_wait)(int sem_handle);
  int (*nacl_irt_sendto)(int sockfd, const void* buffer, size_t len, int flags, const struct nacl_abi_sockaddr* src, size_t addrlen);
  void (*nacl_irt_serve_link_request)(int (*func)(int nexe_fd, const int* obj_file_fds, int obj_file_fd_count));
  int (*nacl_irt_setsockopt)(int sockfd, int level, int optname, const void* optval, size_t optlen);
  int (*nacl_irt_shutdown)(int sockfd, int flags);
  int (*nacl_irt_socket)(int domain, int type, int protocol, int* newfd);
  int (*nacl_irt_socketpair)(int domain, int type, int protocol, int sv[2]);
  int (*nacl_irt_stat)(const char* pathname, struct stat*);
  int (*nacl_irt_symlink)(const char* oldpath, const char* newpath);
  int (*nacl_irt_sysconf)(int name, int* value);
  int (*nacl_irt_thread_create)(void (*start_func)(void), void* stack, void* thread_ptr);
  void (*nacl_irt_thread_exit)(int32_t* stack_flag);
  int (*nacl_irt_thread_nice)(const int nice);
  void* (*nacl_irt_tls_get)(void);
  int (*nacl_irt_tls_init)(void* thread_ptr);
  int (*nacl_irt_truncate)(const char* pathname, nacl_irt_int64_t length);
  int (*nacl_irt_unlink)(const char* pathname);
  int (*nacl_irt_utimes)(const char* filename, const struct timeval* times);
  int (*nacl_irt_write)(int fd, const void* buf, size_t count, size_t* nwrote);
} _irt_syscalls_t;

extern _irt_syscalls_t* g_nacl_irt_syscalls_ptr;
#define _IRT_SYSCALL_PTR  g_nacl_irt_syscalls_ptr

#define __nacl_irt_query (_IRT_SYSCALL_PTR->nacl_irt_query)
#define __nacl_irt_accept (_IRT_SYSCALL_PTR->nacl_irt_accept)
#define __nacl_irt_access (_IRT_SYSCALL_PTR->nacl_irt_access)
#define __nacl_irt_allocate_code_data (_IRT_SYSCALL_PTR->nacl_irt_allocate_code_data)
#define __nacl_irt_bind (_IRT_SYSCALL_PTR->nacl_irt_bind)
#define __nacl_irt_chdir (_IRT_SYSCALL_PTR->nacl_irt_chdir)
#define __nacl_irt_chmod (_IRT_SYSCALL_PTR->nacl_irt_chmod)
#define __nacl_irt_clock (_IRT_SYSCALL_PTR->nacl_irt_clock)
#define __nacl_irt_clock_getres (_IRT_SYSCALL_PTR->nacl_irt_clock_getres)
#define __nacl_irt_clock_gettime (_IRT_SYSCALL_PTR->nacl_irt_clock_gettime)
#define __nacl_irt_close (_IRT_SYSCALL_PTR->nacl_irt_close)
#define __nacl_irt_cond_broadcast (_IRT_SYSCALL_PTR->nacl_irt_cond_broadcast)
#define __nacl_irt_cond_create (_IRT_SYSCALL_PTR->nacl_irt_cond_create)
#define __nacl_irt_cond_destroy (_IRT_SYSCALL_PTR->nacl_irt_cond_destroy)
#define __nacl_irt_cond_signal (_IRT_SYSCALL_PTR->nacl_irt_cond_signal)
#define __nacl_irt_cond_timed_wait_abs (_IRT_SYSCALL_PTR->nacl_irt_cond_timed_wait_abs)
#define __nacl_irt_cond_wait (_IRT_SYSCALL_PTR->nacl_irt_cond_wait)
#define __nacl_irt_connect (_IRT_SYSCALL_PTR->nacl_irt_connect)
#define __nacl_irt_dup (_IRT_SYSCALL_PTR->nacl_irt_dup)
#define __nacl_irt_dup2 (_IRT_SYSCALL_PTR->nacl_irt_dup2)
#define __nacl_irt_dyncode_create (_IRT_SYSCALL_PTR->nacl_irt_dyncode_create)
#define __nacl_irt_dyncode_delete (_IRT_SYSCALL_PTR->nacl_irt_dyncode_delete)
#define __nacl_irt_dyncode_modify (_IRT_SYSCALL_PTR->nacl_irt_dyncode_modify)
#define __nacl_irt_epoll_create (_IRT_SYSCALL_PTR->nacl_irt_epoll_create)
#define __nacl_irt_epoll_ctl (_IRT_SYSCALL_PTR->nacl_irt_epoll_ctl)
#define __nacl_irt_epoll_wait (_IRT_SYSCALL_PTR->nacl_irt_epoll_wait)
#define __nacl_irt_exception_clear_flag (_IRT_SYSCALL_PTR->nacl_irt_exception_clear_flag)
#define __nacl_irt_exception_handler (_IRT_SYSCALL_PTR->nacl_irt_exception_handler)
#define __nacl_irt_exception_stack (_IRT_SYSCALL_PTR->nacl_irt_exception_stack)
#define __nacl_irt_exit (_IRT_SYSCALL_PTR->nacl_irt_exit)
#define __nacl_irt_fchdir (_IRT_SYSCALL_PTR->nacl_irt_fchdir)
#define __nacl_irt_fchmod (_IRT_SYSCALL_PTR->nacl_irt_fchmod)
#define __nacl_irt_fdatasync (_IRT_SYSCALL_PTR->nacl_irt_fdatasync)
#define __nacl_irt_fstat (_IRT_SYSCALL_PTR->nacl_irt_fstat)
#define __nacl_irt_fsync (_IRT_SYSCALL_PTR->nacl_irt_fsync)
#define __nacl_irt_ftruncate (_IRT_SYSCALL_PTR->nacl_irt_ftruncate)
#define __nacl_irt_futex_wait_abs (_IRT_SYSCALL_PTR->nacl_irt_futex_wait_abs)
#define __nacl_irt_futex_wake (_IRT_SYSCALL_PTR->nacl_irt_futex_wake)
#define __nacl_irt_get_random_bytes (_IRT_SYSCALL_PTR->nacl_irt_get_random_bytes)
#define __nacl_irt_getcwd (_IRT_SYSCALL_PTR->nacl_irt_getcwd)
#define __nacl_irt_getdents (_IRT_SYSCALL_PTR->nacl_irt_getdents)
#define __nacl_irt_getpeername (_IRT_SYSCALL_PTR->nacl_irt_getpeername)
#define __nacl_irt_getpid (_IRT_SYSCALL_PTR->nacl_irt_getpid)
#define __nacl_irt_getsockname (_IRT_SYSCALL_PTR->nacl_irt_getsockname)
#define __nacl_irt_getsockopt (_IRT_SYSCALL_PTR->nacl_irt_getsockopt)
#define __nacl_irt_gettod (_IRT_SYSCALL_PTR->nacl_irt_gettod)
#define __nacl_irt_isatty (_IRT_SYSCALL_PTR->nacl_irt_isatty)
#define __nacl_irt_link (_IRT_SYSCALL_PTR->nacl_irt_link)
#define __nacl_irt_list_mappings (_IRT_SYSCALL_PTR->nacl_irt_list_mappings)
#define __nacl_irt_listen (_IRT_SYSCALL_PTR->nacl_irt_listen)
#define __nacl_irt_lstat (_IRT_SYSCALL_PTR->nacl_irt_lstat)
#define __nacl_irt_mkdir (_IRT_SYSCALL_PTR->nacl_irt_mkdir)
#define __nacl_irt_mmap (_IRT_SYSCALL_PTR->nacl_irt_mmap)
#define __nacl_irt_mprotect (_IRT_SYSCALL_PTR->nacl_irt_mprotect)
#define __nacl_irt_munmap (_IRT_SYSCALL_PTR->nacl_irt_munmap)
#define __nacl_irt_mutex_create (_IRT_SYSCALL_PTR->nacl_irt_mutex_create)
#define __nacl_irt_mutex_destroy (_IRT_SYSCALL_PTR->nacl_irt_mutex_destroy)
#define __nacl_irt_mutex_lock (_IRT_SYSCALL_PTR->nacl_irt_mutex_lock)
#define __nacl_irt_mutex_trylock (_IRT_SYSCALL_PTR->nacl_irt_mutex_trylock)
#define __nacl_irt_mutex_unlock (_IRT_SYSCALL_PTR->nacl_irt_mutex_unlock)
#define __nacl_irt_nanosleep (_IRT_SYSCALL_PTR->nacl_irt_nanosleep)
#define __nacl_irt_open (_IRT_SYSCALL_PTR->nacl_irt_open)
#define __nacl_irt_open_resource (_IRT_SYSCALL_PTR->nacl_irt_open_resource)
#define __nacl_irt_pipe (_IRT_SYSCALL_PTR->nacl_irt_pipe)
#define __nacl_irt_poll (_IRT_SYSCALL_PTR->nacl_irt_poll)
#define __nacl_irt_read (_IRT_SYSCALL_PTR->nacl_irt_read)
#define __nacl_irt_readlink (_IRT_SYSCALL_PTR->nacl_irt_readlink)
#define __nacl_irt_recvfrom (_IRT_SYSCALL_PTR->nacl_irt_recvfrom)
#define __nacl_irt_register_block_hooks (_IRT_SYSCALL_PTR->nacl_irt_register_block_hooks)
#define __nacl_irt_rename (_IRT_SYSCALL_PTR->nacl_irt_rename)
#define __nacl_irt_rmdir (_IRT_SYSCALL_PTR->nacl_irt_rmdir)
#define __nacl_irt_sched_yield (_IRT_SYSCALL_PTR->nacl_irt_sched_yield)
#define __nacl_irt_seek (_IRT_SYSCALL_PTR->nacl_irt_seek)
#define __nacl_irt_select (_IRT_SYSCALL_PTR->nacl_irt_select)
#define __nacl_irt_sem_create (_IRT_SYSCALL_PTR->nacl_irt_sem_create)
#define __nacl_irt_sem_destroy (_IRT_SYSCALL_PTR->nacl_irt_sem_destroy)
#define __nacl_irt_sem_post (_IRT_SYSCALL_PTR->nacl_irt_sem_post)
#define __nacl_irt_sem_wait (_IRT_SYSCALL_PTR->nacl_irt_sem_wait)
#define __nacl_irt_sendto (_IRT_SYSCALL_PTR->nacl_irt_sendto)
#define __nacl_irt_serve_link_request (_IRT_SYSCALL_PTR->nacl_irt_serve_link_request)
#define __nacl_irt_setsockopt (_IRT_SYSCALL_PTR->nacl_irt_setsockopt)
#define __nacl_irt_shutdown (_IRT_SYSCALL_PTR->nacl_irt_shutdown)
#define __nacl_irt_socket (_IRT_SYSCALL_PTR->nacl_irt_socket)
#define __nacl_irt_socketpair (_IRT_SYSCALL_PTR->nacl_irt_socketpair)
#define __nacl_irt_stat (_IRT_SYSCALL_PTR->nacl_irt_stat)
#define __nacl_irt_symlink (_IRT_SYSCALL_PTR->nacl_irt_symlink)
#define __nacl_irt_sysconf (_IRT_SYSCALL_PTR->nacl_irt_sysconf)
#define __nacl_irt_thread_create (_IRT_SYSCALL_PTR->nacl_irt_thread_create)
#define __nacl_irt_thread_exit (_IRT_SYSCALL_PTR->nacl_irt_thread_exit)
#define __nacl_irt_thread_nice (_IRT_SYSCALL_PTR->nacl_irt_thread_nice)
#define __nacl_irt_tls_get (_IRT_SYSCALL_PTR->nacl_irt_tls_get)
#define __nacl_irt_tls_init (_IRT_SYSCALL_PTR->nacl_irt_tls_init)
#define __nacl_irt_truncate (_IRT_SYSCALL_PTR->nacl_irt_truncate)
#define __nacl_irt_unlink (_IRT_SYSCALL_PTR->nacl_irt_unlink)
#define __nacl_irt_utimes (_IRT_SYSCALL_PTR->nacl_irt_utimes)
#define __nacl_irt_write (_IRT_SYSCALL_PTR->nacl_irt_write)


int __nacl_abi_errno_to_errno(int err);
#define ENOSYS_IF_NULL(x) if ((x) == NULL) { errno = ENOSYS; return -1; }

__END_DECLS
#endif
